# generated by datamodel-codegen:
#   filename:  open_url.json

from __future__ import annotations

from pydantic import BaseModel


class OpenUrl(BaseModel):
    """
    Open URL
    """

    url: str
