# generated by datamodel-codegen:
#   filename:  disk.json

from __future__ import annotations

from typing import Optional

from pydantic import BaseModel, Extra, Field


class LastUpdated(BaseModel):
    """
    Last updated
    """

    class Config:
        extra = Extra.allow

    devices: float
    io_counters_read_count: Optional[float] = None
    io_counters_write_count: Optional[float] = None
    io_counters_read_bytes: Optional[float] = None
    io_counters_write_bytes: Optional[float] = None
    io_counters_read_time: Optional[float] = None
    io_counters_write_time: Optional[float] = None
    partitions: float


class Disk(BaseModel):
    """
    Disk
    """

    class Config:
        extra = Extra.allow

    id: Optional[str] = Field(None, description="Event ID")
    devices: list
    io_counters_read_count: Optional[int] = None
    io_counters_write_count: Optional[int] = None
    io_counters_read_bytes: Optional[int] = None
    io_counters_write_bytes: Optional[int] = None
    io_counters_read_time: Optional[int] = None
    io_counters_write_time: Optional[int] = None
    partitions: list
    last_updated: Optional[LastUpdated] = Field(None, description="Last updated")
