"""
Provides systembridgeshared version information.
"""

# This file is auto-generated! Do not edit!
# Use `python -m incremental.update systembridgeshared` to change this file.

from incremental import Version

__version__ = Version("systembridgeshared", 3, 6, 3, dev=1)
__all__ = ["__version__"]
