# generated by datamodel-codegen:
#   filename:  media_files.json

from __future__ import annotations

from typing import Optional

from pydantic import BaseModel


class File(BaseModel):
    """
    Media File
    """

    name: str
    path: str
    fullpath: str
    size: int
    last_accessed: float
    created: float
    modified: float
    is_directory: bool
    is_file: bool
    is_link: bool
    mime_type: Optional[str] = None


class MediaFiles(BaseModel):
    """
    Media Files
    """

    files: list[File]
    path: str
