# generated by datamodel-codegen:
#   filename:  action.json

from __future__ import annotations

from typing import Any, Optional

from pydantic import BaseModel


class Action(BaseModel):
    """
    Action
    """

    command: str
    data: Optional[dict[str, Any]] = None
    label: Optional[str] = None
