# generated by datamodel-codegen:
#   filename:  media_get_files.json

from __future__ import annotations

from typing import Optional

from pydantic import BaseModel


class MediaGetFiles(BaseModel):
    """
    Get Files
    """

    base: str
    path: Optional[str] = None
