# generated by datamodel-codegen:
#   filename:  get_setting.json

from __future__ import annotations

from pydantic import BaseModel


class GetSetting(BaseModel):
    """
    Get Setting
    """

    setting: str
